/*
 * Access a filesystem.
 *
 * Copyright (C) 2019, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <unistd.h>

#include <ext2fs/ext2fs.h>

#include "ops.h"
#include "session.h"



/* Help message. */

char help_text[] = "\
Usage: %s [ <options> ] <image file> <operation> <filename>...\n\
\n\
File ownership options:\n\
\n\
  -g GID        Set group identifier for new files\n\
  -u UID        Set user identifier for new files\n\
\n\
File permission options:\n\
\n\
  -m MASK       Set mode/permissions mask for new directories\n\
\n\
Transfer operations:\n\
\n\
  copy-in       Copy files into a directory within the image\n\
  copy-out      Copy files from the image into a directory\n\
\n\
Image operations:\n\
\n\
  ls            List files and directories within the image\n\
  mkdir         Make directories within the image\n\
  rm            Remove non-directory objects from the image\n\
  rmdir         Remove directories from the image\n\
  stat          Show statistics for files and directories\n\
\n\
Script operations:\n\
\n\
  script        Read operations from a script file\n\
";

/* Operations exposed by the program. */

struct operation operations[] = {
    {"copy-in", copy_in},
    {"copy-out", copy_out},
    {"ls", list},
    {"mkdir", make_dirs},
    {"rm", remove_non_dirs},
    {"rmdir", remove_dirs},
    {"script", run_script},
    {"stat", stat_objects},
    {NULL, NULL},
    };

/* Main program. */

int main(int argc, char *argv[])
{
    int flags = EXT2_FLAG_RW; // | EXT2_FLAG_SOFTSUPP_FEATURES | EXT2_FLAG_64BITS;
    ext2_filsys fs = NULL;
    errcode_t retval;
    int exitcode = 0;

    /* Program argument details. */

    char **args;
    char *fsname, *filename;
    int num_args;
    enum op_results op_result;

    /* Parse program options and initialise the argument details. */

    if (parse_options(argc, argv))
        return 1;

    args = &argv[optind];
    num_args = argc - optind;

    if (num_args < 3)
    {
        fprintf(stderr, help_text, argv[0]);
        return 1;
    }

    /* Open the filesystem image using the POSIX file access mechanism. */

    fsname = args[0];

    retval = ext2fs_open(fsname, flags, 0, 0, unix_io_manager, &fs);
    if (retval)
    {
        fprintf(stderr, "Could not open filesystem: %s\n", fsname);
        return 1;
    }

    //fs->default_bitmap_type = EXT2FS_BMAP64_RBTREE;

    retval = ext2fs_read_bitmaps(fs);
    if (retval)
    {
        fprintf(stderr, "Could not read bitmaps: %s\n", fsname);
        return 1;
    }

    /* Perform the requested operation. */

    op_result = run_operation(fs, args[1], num_args - 2, &args[2]);
    exitcode = handle_op_result(args[1], op_result);

    /* Close the filesystem image. */

    retval = ext2fs_flush(fs);
    if (retval)
    {
        fprintf(stderr, "Error flushing filesystem: %s\n", fsname);
        exitcode = 1;
    }

    retval = ext2fs_close(fs);
    if (retval)
    {
        fprintf(stderr, "Error closing filesystem: %s\n", fsname);
        exitcode = 1;
    }

    ext2fs_free(fs);
    return exitcode;
}

/* vim: tabstop=4 expandtab shiftwidth=4
*/
