/*
 * Program memory segment support.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/err.h>

#include <ipc/mem_ipc.h>
#include <mem/memory_utils.h>

#include <stdio.h>
#include <string.h>

#include "segment.h"



/* Initialise a memory segment. */

Segment::Segment(l4_addr_t base, offset_t size, l4re_rm_flags_t flags,
                 offset_t file_offset, offset_t file_contents)

: _base(base), _size(size), _flags(flags), _file_offset(file_offset),
  _file_contents(file_contents)
{
  _region_base = trunc(_base, L4_PAGESIZE);
  _region_offset = _base - _region_base;
  _region_size = round(_size, L4_PAGESIZE);
}

/* Return the address of allocated memory. */

char *Segment::address()
{
  return _buf;
}

/* Return the size of the allocated memory. */

offset_t Segment::size()
{
  return _size;
}

/* Allocate a region for the segment. */

long Segment::allocate()
{
  /* Make regions writable if they need to be filled. */

  l4re_rm_flags_t allocation_flags = _flags;

  if (_file_contents)
    allocation_flags |= L4RE_RM_F_W;

  return ipc_allocate_align(_size, L4RE_RM_F_SEARCH_ADDR | allocation_flags,
                            L4_PAGESHIFT, (void **) &_buf, &_ds);
}

/* Fill a segment region with file content. */

long Segment::fill(file_t *file)
{
  if (!_file_contents)
    return L4_EOK;

  memset(_buf, 0, _region_size);
  client_seek(file, _file_offset, SEEK_SET);
  offset_t nread = client_read(file, _buf + _region_offset, _file_contents);

  if (nread < _file_contents)
    return -L4_EIO;
  else
    return L4_EOK;
}

/* Define and return the mapped region for the segment. */

MappedRegion &Segment::region()
{
  _region = MappedRegion((l4_addr_t) _buf, page_order(_region_size), _flags, _region_base);
  return _region;
}

/* Return the mapped address corresponding to the given address. */

l4_addr_t Segment::region_address(char *address)
{
  return (l4_addr_t) ((address - _buf) + (char *) _region_base);
}

l4_addr_t Segment::region_address(l4_addr_t address)
{
  return (address - (l4_addr_t) _buf) + _region_base;
}

/* vim: tabstop=2 expandtab shiftwidth=2
*/
